#ifndef __XLIC_HEX_HPP__
#define __XLIC_HEX_HPP__

#include <vector>
#include <string>
#include <cstddef>
#include <cstdint>
#include <cstring>
#include "config.h"

struct hex {

	static std::vector<uint8_t> decode(const char * input);
	static std::vector<uint8_t> decode(
		const void * input,
		const size_t length);
	static std::vector<uint8_t> decode(const std::string & input);
	static size_t decode(
		void * output,
		const void * input,
		const size_t length) throw();

	static std::string encode(const char * input);
	static std::string encode(
		const void * input,
		const size_t length);
	static std::string encode(const std::string & input);
	template<class T> static std::string encode(const std::vector<T> & input);
	static size_t encode(
		void * output,
		const void * input,
		const size_t length) throw();

	static bool valid(const char * input) throw();
	static bool valid(const std::string & input) throw();
	static bool valid(
		const void * input,
		const size_t length) throw();

	static size_t decoded_length(const size_t length) throw();

	static size_t encoded_length(const size_t length) throw();
};

inline size_t hex::decoded_length(const size_t length) throw() {
	return length / 2;
}

inline size_t hex::encoded_length(const size_t length) throw() {
	return length * 2;
}

inline std::vector<uint8_t> hex::decode(const char * input) {
	return decode(input, strlen(input));
}

inline std::vector<uint8_t> hex::decode(
	const void * input,
	const size_t length
) {
	std::vector<uint8_t> output(decoded_length(length));
	decode(output.data(), input, length);
	return output;
}

inline std::vector<uint8_t> hex::decode(const std::string & input) {
	return decode(input.data(), sizeof(std::string::value_type) * input.length());
}

inline std::string hex::encode(const char * input) {
	return encode(input, strlen(input));
}

inline std::string hex::encode(const std::string & input) {
	return encode(input.data(), sizeof(std::string::value_type) * input.length());
}

template<class T>
inline std::string hex::encode(const std::vector<T> & input) {
	return encode(input.data(), sizeof(T) * input.size());
}

inline std::string hex::encode(
	const void * input,
	const size_t length
) {
	std::string output(encoded_length(length), 0);
	encode(&output[0], input, length);
	return output;
}

inline bool hex::valid(const char * input) throw() {
	return valid(input, strlen(input));
}

inline bool hex::valid(const std::string & input) throw() {
	return valid(input.data(), sizeof(std::string::value_type) * input.length());
}

#endif
