#ifndef __MICROTIME_HPP__
#define __MICROTIME_HPP__

#include <cstdint>
#include <string>
#include <cstring>
#include "util.hpp"
#include "config.h"

class microtime_t {
public:

	static const microtime_t invalid;

	microtime_t();
	explicit microtime_t(const uint64_t input) throw();

	static microtime_t from_string(const char * input) throw();
	static microtime_t from_string(
		const void * input,
		const size_t length) throw();

	std::string to_string() const;

	uint64_t value() const throw();

	microtime_t & operator=(const uint64_t input) throw();

	bool operator==(const microtime_t & microtime) throw();
	bool operator==(const uint64_t value) throw();

	bool operator!=(const microtime_t & microtime) throw();
	bool operator!=(const uint64_t value) throw();

private:

	uint64_t m_value;
};

inline microtime_t::microtime_t()
: m_value(invalid.m_value) {}

inline microtime_t::microtime_t(const uint64_t input) throw()
: m_value(input) {}

inline microtime_t microtime_t::from_string(const char * input) throw() {
	return from_string(input, strlen(input));
}

inline microtime_t microtime_t::from_string(
	const void * input,
	const size_t length
) throw() {
	return microtime_t(strntoull(input, length));
}

inline uint64_t microtime_t::value() const throw() {
	return m_value;
}

inline microtime_t & microtime_t::operator=(const uint64_t input) throw() {
	m_value = input;
	return *this;
}

inline bool microtime_t::operator==(const microtime_t & microtime) throw() {
	return (m_value == microtime.m_value);
}

inline bool microtime_t::operator==(const uint64_t value) throw() {
	return (m_value == value);
}

inline bool microtime_t::operator!=(const microtime_t & microtime) throw() {
	return (m_value != microtime.m_value);
}

inline bool microtime_t::operator!=(const uint64_t value) throw() {
	return (m_value != value);
}

#endif
