#include <system_error>
#include "license_file.hpp"
#include "lockfile.hpp"
#include "base64.hpp"
#include "globals.hpp"
#include "string_ref.hpp"

extern "C" int xlic_remove_license(
	const char * filename,
	const char * url,
	const char * base64_license_key
) {
	try {
		license_key_t license_key = license_key_t::from_base64(base64_license_key);
		if (license_key == license_key_t::invalid) {
			throw std::logic_error("Invalid license key provided");
		}
		lockfile lock(XLIC_LOCKFILE);
		if (lock.acquire()) {
			license_file license_bin(filename);
			if (license_bin.remove((string_ref(url) + "unregister").data(), license_key)) {
				license_bin.save(filename);
				return EXIT_SUCCESS;
			}
			fprintf(stderr, "No such license exists on this system\n");
		} else {
			fprintf(stderr, "License file is locked, cannot perform operation at this time\n");
			return EBUSY;
		}
	} catch (std::system_error & e) {
		fprintf(stderr, "Error: %s\n", e.what());
		return e.code().value();
	} catch (std::exception & e) {
		fprintf(stderr, "Error: %s\n", e.what());
	}
	return EINVAL;
}
