#ifndef __SIGNATURE_HPP__
#define __SIGNATURE_HPP__

#include <cstdint>
#include <cstring>
#include <xorcom/licensing.h>
#include "hex.hpp"
#include "base64.hpp"
#include "config.h"

class signature_t : public xlic_signature {
public:

	static const signature_t invalid;

	signature_t();
	explicit signature_t(const xlic_signature & input) throw();

	static signature_t from_hex(const char * input) throw();
	static signature_t from_hex(
		const void * input,
		const size_t length) throw();

	std::string to_base64() const;

	std::string to_hex() const;

	signature_t & operator=(const xlic_signature & input) throw();

	bool operator==(const signature_t & signature) throw();
	bool operator==(const xlic_signature & signature) throw();

	bool operator!=(const signature_t & signature) throw();
	bool operator!=(const xlic_signature & signature) throw();
};

inline signature_t::signature_t()
: xlic_signature(invalid) {}

inline signature_t::signature_t(const xlic_signature & input) throw()
: xlic_signature(input) {}

inline signature_t signature_t::from_hex(const char * input) throw() {
	return from_hex(input, strlen(input));
}

inline signature_t signature_t::from_hex(
	const void * input,
	const size_t length
) throw() {
	if (hex::decoded_length(length) == sizeof(xlic_signature) && hex::valid(input, length)) {
		signature_t tmp;
		hex::decode(tmp.data, input, length);
		return tmp;
	}
	return invalid;
}

inline std::string signature_t::to_base64() const {
	return base64::encode(data, sizeof(xlic_signature));
}

inline std::string signature_t::to_hex() const {
	return hex::encode(data, sizeof(xlic_signature));
}

inline signature_t & signature_t::operator=(const xlic_signature & signature) throw() {
	memcpy(data, signature.data, sizeof(xlic_signature));
	return *this;
}

inline bool signature_t::operator==(const signature_t & signature) throw() {
	return (memcmp(data, signature.data, sizeof(xlic_signature)) == 0);
}

inline bool signature_t::operator==(const xlic_signature & signature) throw() {
	return (memcmp(data, signature.data, sizeof(xlic_signature)) == 0);
}

inline bool signature_t::operator!=(const signature_t & signature) throw() {
	return (memcmp(data, signature.data, sizeof(xlic_signature)) != 0);
}

inline bool signature_t::operator!=(const xlic_signature & signature) throw() {
	return (memcmp(data, signature.data, sizeof(xlic_signature)) != 0);
}

#endif
