#ifndef __TIMESTAMP_HPP__
#define __TIMESTAMP_HPP__

#include <cstdint>
#include <string>
#include <cstring>
#include "util.hpp"
#include "config.h"

class timestamp_t {
public:

	static const timestamp_t invalid;

	timestamp_t() throw();
	explicit timestamp_t(const uint32_t value) throw();

	static timestamp_t from_string(const char * input) throw();
	static timestamp_t from_string(
		const void * input,
		const size_t length) throw();

	static timestamp_t now() throw();

	std::string to_string() const;

	uint32_t value() const throw();

	timestamp_t & operator=(const uint32_t input) throw();

	bool operator==(const timestamp_t & timestamp) const throw();
	bool operator==(const uint32_t value) const throw();

	bool operator!=(const timestamp_t & timestamp) const throw();
	bool operator!=(const uint32_t value) const throw();

	bool operator<(const timestamp_t & timestamp) const throw();
	bool operator<(const uint32_t value) const throw();

	bool operator>(const timestamp_t & timestamp) const throw();
	bool operator>(const uint32_t value) const throw();

	bool operator<=(const timestamp_t & timestamp) const throw();
	bool operator<=(const uint32_t value) const throw();

	bool operator>=(const timestamp_t & timestamp) const throw();
	bool operator>=(const uint32_t value) const throw();

private:

	uint32_t m_value;
};

inline timestamp_t::timestamp_t() throw()
: m_value(invalid.m_value) {}

inline timestamp_t::timestamp_t(const uint32_t value) throw()
: m_value(value) {}

inline timestamp_t timestamp_t::from_string(const char * input) throw() {
	return from_string(input, strlen(input));
}

inline timestamp_t timestamp_t::from_string(
	const void * input,
	const size_t length
) throw() {
	return timestamp_t(strntoull(input, length));
}

inline timestamp_t timestamp_t::now() throw() {
	return timestamp_t(time(nullptr));
}

inline uint32_t timestamp_t::value() const throw() {
	return m_value;
}

inline bool timestamp_t::operator==(const timestamp_t & timestamp) const throw() {
	return (m_value == timestamp.m_value);
}

inline bool timestamp_t::operator==(const uint32_t value) const throw() {
	return (m_value == value);
}

inline bool timestamp_t::operator!=(const timestamp_t & timestamp) const throw() {
	return (m_value != timestamp.m_value);
}

inline bool timestamp_t::operator!=(const uint32_t value) const throw() {
	return (m_value != value);
}

inline bool timestamp_t::operator<(const timestamp_t & timestamp) const throw() {
	return (m_value < timestamp.m_value);
}

inline bool timestamp_t::operator<(const uint32_t value) const throw() {
	return (m_value < value);
}

inline bool timestamp_t::operator>(const timestamp_t & timestamp) const throw() {
	return (m_value > timestamp.m_value);
}

inline bool timestamp_t::operator>(const uint32_t value) const throw() {
	return (m_value > value);
}

inline bool timestamp_t::operator<=(const timestamp_t & timestamp) const throw() {
	return (m_value <= timestamp.m_value);
}

inline bool timestamp_t::operator<=(const uint32_t value) const throw() {
	return (m_value <= value);
}

inline bool timestamp_t::operator>=(const timestamp_t & timestamp) const throw() {
	return (m_value >= timestamp.m_value);
}

inline bool timestamp_t::operator>=(const uint32_t value) const throw() {
	return (m_value >= value);
}

inline bool operator==(
	const uint32_t lhs,
	const timestamp_t & rhs
) throw() {
	return (rhs == lhs);
}

inline bool operator!=(
	const uint32_t lhs,
	const timestamp_t & rhs
) throw() {
	return (rhs != lhs);
}

inline bool operator<(
	const uint32_t lhs,
	const timestamp_t & rhs
) throw() {
	return (rhs > lhs);
}

inline bool operator>(
	const uint32_t lhs,
	const timestamp_t & rhs
) throw() {
	return (rhs < lhs);
}

inline bool operator<=(
	const uint32_t lhs,
	const timestamp_t & rhs
) throw() {
	return (rhs >= lhs);
}

inline bool operator>=(
	const uint32_t lhs,
	const timestamp_t & rhs
) throw() {
	return (rhs <= lhs);
}

#endif
